#!/bin/bash

# This script was adapted from the Nagios Core Ubuntu Quickstart guide:
# https://assets.nagios.com/downloads/nagioscore/docs/nagioscore/4/en/quickstart-ubuntu.html

set -ex

function add_required_packages() {
  apt-get update -qq
  # Required Packages
  apt-get install -y \
    apache2 \
    apache2-utils \
    build-essential \
    libapache2-mod-php5 \
    libgd2-xpm-dev \
    libssl-dev \
    php5 \
    php5-gd \
    pynag \
    python-pip \
    python-pynag \
    unzip \
    wget
  pip install charmhelpers
}

function create_account_information() {
  # 1) Create Account Information
  if grep -q '^nagios:' /etc/passwd; then
    juju-log "The nagios user already exists.";
  else
    # Create a new nagios user account
    /usr/sbin/useradd -m -s /bin/bash nagios
  fi
  # Create a new nagcmd group for external commands to the web interface
  /usr/sbin/groupadd -f nagcmd
  /usr/sbin/usermod -a -G nagcmd nagios
  /usr/sbin/usermod -a -G nagcmd www-data
}

function download_nagios() {
  local nagios_core_uri = $1
  local nagios_core_sha1 = $2
  local nagios_core_filename=`basename ${nagios_core_uri}`

  if [[ ! -f ${CHARM_DIR}/files/${nagios_core_filename} ]]; then
    # 2) Download Nagios core
    wget -nv -O ${CHARM_DIR}/files/${nagios_core_filename} ${nagios_core_uri}
    SUM=$(sha1sum ${CHARM_DIR}/files/${nagios_core_filename} | cut -d' ' -f 1)
    # Verify the sha1sum of the nagios core matches what is expected.
    if [[ ${SUM} != ${nagios_core_sha1} ]]; then
      status-set blocked "The sha1sum for ${nagios_core_filename} did not match, verify the value and retry the set config operation"
      exit 0
    fi
  fi
}

function download_nagos_plugins() {
  local nagios_plugins_uri = $1
  local nagios_plugins_sha1 = $2
  local nagios_plugins_filename=`basename ${nagios_plugins_uri}`

  if [[ ! -f ${CHARM_DIR}/files/${nagios_plugins_filename} ]]; then
    # 2) Download Nagios Plugins
    wget -nv -O ${CHARM_DIR}/files/${nagios_plugins_filename} ${nagios_plugins_uri}
    SUM=$(sha1sum ${CHARM_DIR}/files/${nagios_plugins_filename} | cut -d' ' -f 1)
    # Verify the sha1sum of the nagios plugins matches what is expected.
    if [[ ${SUM} != ${nagios_plugins_sha1} ]]; then
      status-set blocked "The sha1sum for ${nagios_plugins_filename} did not match, verify the value and retry the set config operation"
      exit 0
    fi
  fi
}

function install_nagios_core() {
  local nagios_core_filename = $1
  local nagios_admin_user = $2
  local nagios_admin_passwd = $3

  # Extract the Nagios source code tarball.
  tar -xvzf ${CHARM_DIR}/files/${nagios_core_filename} -C /tmp
  # 3) Compile and Install Nagios
  local nagios_core_dir=`basename /tmp/nagios-*`
  cd /tmp/${nagios_core_dir}
  # Run the Nagios configure script, passing the name of the group created.
  ./configure --with-command-group=nagcmd --with-httpd_conf=/etc/apache2/conf-available
  # Compile the Nagios source code.
  make all
  # Install binaries, init script, sample config files and set permissions on the external command directory.
  make install
  make install-init
  make install-config
  make install-commandmode

  # 5) Configure the Web Interface
  make install-webconf
  # Install the Nagios web config file in the Apache conf-enabled directory.
  /usr/sbin/a2enconf nagios
  # Create an administrator account for logging into the Nagios web interface.
  htpasswd -b -c /usr/local/nagios/etc/htpasswd.users ${nagios_admin_user} ${nagios_admin_passwd}
  # Enable the cgi apache module.
  /usr/sbin/a2enmod cgi
  # Restart Apache to make the new settings take effect.
  service apache2 restart

  # Don't start Nagios yet - there's still more that needs to be done...
  cd ${CHARM_DIR}
  rm -rf /tmp/${nagios_core_dir}
}

function update_password() {
  local nagios_admin_user=$1
  local nagios_admin_passwd=$2
  htpasswd -b -c /usr/local/nagios/etc/htpasswd.users ${nagios_admin_user} ${nagios_admin_passwd}
  service nagios restart
}

function install_nagios_plugins() {
  local nagios_plugins_filename = $1

  # Extract the Nagios Plugins source code tarball.
  tar -xvzf ${CHARM_DIR}/files/${nagios_plugins_filename} -C /tmp
  # 6) Compile and Install the Nagios Plugins
  local nagios_plugin_dir=`basename /tmp/nagios-plugins-*`
  cd /tmp/${nagios_plugin_dir}
  ./configure --with-nagios-user=nagios --with-nagios-group=nagios
  # Compile and install the plugins.
  make
  make install
  cd ${CHARM_DIR}
  rm -rf /tmp/${nagios_plugin_dir}
}
